@@ -17,6 +17,9 @@ if False:  # pragma: nocover
 
 USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')
 
+TypeFlagsForType = List['FlagBase']
+TypeFlagsForTypes = Dict[Type[models.Model], TypeFlagsForType]
+
 
 class FlagBase(models.Model):
     """Base class for flag models.
@@ -63,19 +66,22 @@ class FlagBase(models.Model):
     def get_flags_for_types(
             cls,
             mdl_classes: List[Type[models.Model]],
+            *,
             user: 'User' = None,
             status: int = None,
-            allow_empty: bool = True
+            allow_empty: bool = True,
+            with_objects: bool = False,
 
-    ) -> Dict[Type[models.Model], Union[Dict[int, 'FlagBase'], Tuple]]:
+    ) -> TypeFlagsForTypes:
         """Returns a dictionary with flag objects associated with the given model classes (types).
         The dictionary is indexed by model classes.
-        Each dict entry contains a list of associated flag objects.
+        Each dict entry contains a list of associated flags.
 
-        :param mdl_classes:
-        :param user:
-        :param status:
+        :param mdl_classes: Types to get flags for.
+        :param user: User filter,
+        :param status: Status filter
         :param allow_empty: Flag. Include results for all given types, even those without associated flags.
+        :param with_objects: Whether to fetch the flagged objects along with the flags.
 
         """
         if not mdl_classes or (user and not user.id):
@@ -83,9 +89,15 @@ class FlagBase(models.Model):
 
         types_for_models = ContentType.objects.get_for_models(*mdl_classes, for_concrete_models=False)
         filter_kwargs = {'content_type__in': types_for_models.values()}
-        update_filter_dict(filter_kwargs, user, status)
+        update_filter_dict(filter_kwargs, user=user, status=status)
+
+        flags = cls.objects.filter(**filter_kwargs)
+
+        if with_objects:
+            flags = flags.prefetch_related('linked_object')
+
+        flags = flags.order_by('-time_created')
 
-        flags = cls.objects.filter(**filter_kwargs).order_by('-time_created')
         flags_dict = defaultdict(list)
 
         for flag in flags:
@@ -101,7 +113,7 @@ class FlagBase(models.Model):
                 result[mdl_cls] = flags_dict[content_type_id]
 
             elif allow_empty:
-                result[mdl_cls] = tuple()
+                result[mdl_cls] = []
 
         return result
 
@@ -109,10 +121,11 @@ class FlagBase(models.Model):
     def get_flags_for_objects(
             cls,
             objects_list: Union[QuerySet, Sequence],
+            *,
             user: 'User' = None,
             status: int = None
 
-    ) -> Dict[int, Union[Dict[int, 'FlagBase'], Tuple]]:
+    ) -> Dict[int, TypeFlagsForType]:
         """Returns a dictionary with flag objects associated with the given model objects.
         The dictionary is indexed by objects IDs.
         Each dict entry contains a list of associated flag objects.
@@ -134,7 +147,7 @@ class FlagBase(models.Model):
             # Consider this list homogeneous.
             'content_type': ContentType.objects.get_for_model(objects_list[0], for_concrete_model=False)
         }
-        update_filter_dict(filter_kwargs, user, status)
+        update_filter_dict(filter_kwargs, user=user, status=status)
 
         flags = cls.objects.filter(**filter_kwargs)
         flags_dict = defaultdict(list)
@@ -145,11 +158,7 @@ class FlagBase(models.Model):
         result = {}
 
         for obj in objects_list:
-            if obj.pk in flags_dict:
-                result[obj.pk] = flags_dict[obj.pk]
-
-            else:
-                result[obj.pk] = tuple()
+            result[obj.pk] = flags_dict.get(obj.pk, [])
 
         return result
 
@@ -173,35 +182,63 @@ class ModelWithFlag(models.Model):
         abstract = True
 
     @classmethod
-    def get_flags_for_types(
+    def get_flags_for_type(
             cls,
-            mdl_classes: List[Type[models.Model]],
+            mdl_classes: List[Type[models.Model]] = None,
+            *,
             user: 'User' = None,
             status: int = None,
-            allow_empty: bool = True
+            allow_empty: bool = True,
+            with_objects: bool = False,
 
-    ) -> Dict[Type[models.Model], Union[Dict[int, 'FlagBase'], Tuple]]:
-        """Returns a dictionary with flag objects associated with the given model classes (types).
+    ) -> Union[TypeFlagsForTypes, TypeFlagsForType]:
+        """Returns a dictionary with flag objects associated with
+        the given model classes (types) if mdl_classes is given.
         The dictionary is indexed by model classes.
         Each dict entry contains a list of associated flag objects.
 
-        :param mdl_classes:
-        :param user:
-        :param status:
+        If mdl_classes is not given, returns a list of associated
+        flag objects for this very class.
+
+        :param mdl_classes: Types to get flags for. If not set the current class is used.
+        :param user: User filter,
+        :param status: Status filter
         :param allow_empty: Flag. Include results for all given types, even those without associated flags.
+        :param with_objects: Whether to fetch the flagged objects along with the flags.
 
         """
         model: FlagBase = get_model_class_from_string(MODEL_FLAG)
-        return model.get_flags_for_types(mdl_classes, user=user, status=status, allow_empty=allow_empty)
+
+        single_type = False
+        if mdl_classes is None:
+            mdl_classes = [cls]
+            single_type = True
+            allow_empty = True
+
+        result = model.get_flags_for_types(
+            mdl_classes,
+            user=user,
+            status=status,
+            allow_empty=allow_empty,
+            with_objects=with_objects,
+        )
+
+        if single_type:
+            result = result[cls]
+
+        return result
+
+    get_flags_for_types = get_flags_for_type  # alias
 
     @classmethod
     def get_flags_for_objects(
             cls,
             objects_list: Union[QuerySet, Sequence],
+            *,
             user: 'User' = None,
             status: int = None
 
-    ) -> Dict[int, Union[Dict[int, 'FlagBase'], Tuple]]:
+    ) -> Dict[int, TypeFlagsForType]:
         """Returns a dictionary with flag objects associated with the given model objects.
         The dictionary is indexed by objects IDs.
         Each dict entry contains a list of associated flag objects.
@@ -214,7 +251,7 @@ class ModelWithFlag(models.Model):
         model: FlagBase = get_model_class_from_string(MODEL_FLAG)
         return model.get_flags_for_objects(objects_list, user=user, status=status)
 
-    def get_flags(self, user: 'User' = None, status: int = None) -> Union[QuerySet, Sequence[FlagBase]]:
+    def get_flags(self, user: 'User' = None, *, status: int = None) -> Union[QuerySet, Sequence[FlagBase]]:
         """Returns flags for the object optionally filtered by status.
 
         :param user: Optional user filter
@@ -222,10 +259,10 @@ class ModelWithFlag(models.Model):
 
         """
         filter_kwargs = {}
-        update_filter_dict(filter_kwargs, user, status)
+        update_filter_dict(filter_kwargs, user=user, status=status)
         return self.flags.filter(**filter_kwargs).all()
 
-    def set_flag(self, user: 'User', note: str = None, status: int = None) -> Optional[FlagBase]:
+    def set_flag(self, user: 'User', *, note: str = None, status: int = None) -> Optional[FlagBase]:
         """Flags the object.
 
         :param user:
@@ -256,7 +293,7 @@ class ModelWithFlag(models.Model):
 
         return flag
 
-    def remove_flag(self, user: 'User' = None, status: int = None):
+    def remove_flag(self, user: 'User' = None, *, status: int = None):
         """Removes flag(s) from the object.
 
         :param user: Optional user filter
@@ -267,10 +304,10 @@ class ModelWithFlag(models.Model):
             'content_type': ContentType.objects.get_for_model(self),
             'object_id': self.id
         }
-        update_filter_dict(filter_kwargs, user, status)
+        update_filter_dict(filter_kwargs, user=user, status=status)
         get_flag_model().objects.filter(**filter_kwargs).delete()
 
-    def is_flagged(self, user: 'User' = None, status: int = None) -> int:
+    def is_flagged(self, user: 'User' = None, *, status: int = None) -> int:
         """Returns a number of times the object is flagged by a user.
 
         :param user: Optional user filter
@@ -281,11 +318,11 @@ class ModelWithFlag(models.Model):
             'content_type': ContentType.objects.get_for_model(self),
             'object_id': self.id,
         }
-        update_filter_dict(filter_kwargs, user, status)
+        update_filter_dict(filter_kwargs, user=user, status=status)
         return self.flags.filter(**filter_kwargs).count()
 
 
-def update_filter_dict(d: dict, user: Optional['User'], status: Optional[int]):
+def update_filter_dict(d: dict, *, user: Optional['User'], status: Optional[int]):
     """Helper. Updates filter dict for a queryset.
 
     :param d:
